@@ -2,5 +2,6 @@ class Spinach::Features::DashboardShortcuts < Spinach::FeatureSteps
   include SharedAuthentication
   include SharedPaths
   include SharedProject
-  include SharedActiveTab
+  include SharedSidebarActiveTab
+  include SharedShortcuts
 end
