@@ -15,5 +15,13 @@ FactoryGirl.define do
     trait :private do
       visibility_level Gitlab::VisibilityLevel::PRIVATE
     end
+
+    trait :access_requestable do
+      request_access_enabled true
+    end
+
+    trait :nested do
+      parent factory: :group
+    end
   end
 end
