@@ -37,6 +37,16 @@ describe GroupLabel, models: true do
       end
     end
 
+    context 'cross-project' do
+      let(:namespace) { build_stubbed(:namespace) }
+      let(:source_project) { build_stubbed(:empty_project, name: 'project-1', namespace: namespace) }
+      let(:target_project) { build_stubbed(:empty_project, name: 'project-2', namespace: namespace) }
+
+      it 'returns a String reference to the object' do
+        expect(label.to_reference(source_project, target_project)).to eq %(project-1~#{label.id})
+      end
+    end
+
     context 'using invalid format' do
       it 'raises error' do
         expect { label.to_reference(format: :invalid) }
